/*
   program : backproject.c

   $Source: /home/yee/C/RCS/angle.c,v $
   $Author: yee $
   $Date: 1999/01/09 17:18:26 $
   $Revision: 1.28 $

   This file contains the code implementing equiangular fan beam
   backprojection.  It is called from code in backproject.c

   For efficiency, some variables in this file are passed between functions
   using static variables.

   Compile Options : none

   Runtime Parameters : none

   Assumptions : none known

   Bugs : none known

   To Do : nothing

   $Log: angle.c,v $
   Revision 1.28  1999/01/09 17:18:26  yee
   documentation fix.

   Revision 1.25  1998/11/27 04:21:00  yee
   bug fix in AngleProjectionFilterWeight().

   Revision 1.19  1997/11/09 20:58:30  yee
   eliminated DetermineRay().

   Copyright 1997 by Conway Yee.  All rights reserved.
 */

char angle[] = "$Id: angle.c,v 1.28 1999/01/09 17:18:26 yee Exp $"; 
#include <math.h>

#include "codes.h"
#include "style.h"
#include "ImageHeader.h"
#include "backproject.h"
#include "recon.h"

PRIVATE double l2;      /* square distance of A+B==L2 declared static since
                           it is calculated in AngleBackprojectPoint() and
                           reused in L2(). */
 
/*****************************************************************************/
/*****************************************************************************/

/* this function weights the backprojection filter for equi-angular
   fan beam data.  It is called by the generic filter code in filter.c */

#ifdef RECON_ANGLE_FAN
PUBLIC double AngleBackprojectionFilterWeight
(
 int ndim,           /* dimensionality of data array */
 int i,              /* element number */
 double *tau         /* degree of freedom, distance between elements */
)
{ /* AngleProjectionFilterWeight() */
                   /* free[0] = tau, distance between elements (in radians) */
  return(0.5/pow(SINC(i*(*tau)),2.0));
} /* AngleProjectionFilterWeight() */
#endif RECON_ANGLE_FAN

/*****************************************************************************/

/* this function weights the projections for equi-angular fan beam data */

#ifdef RECON_ANGLE_FAN
PUBLIC double AngleProjectionWeight
(
 int ndim,           /* dimensionality of data array */
 int i,              /* element number */
 double *FloatFree   /* degrees of freedom */
)
{ /* AngleProjectionWeight() */
                   /* FloatFree[0] = D, distance between focal point and center
                                     of rotation */
                   /* FloatFree[1] = tau, angular distance between elements */
   return(FloatFree[0]*cos(i*FloatFree[1]));
} /* AngleProjectionWeight() */
#endif RECON_ANGLE_FAN

/*****************************************************************************/

/* this function calculates the point from which a pixel is backprojected
   in units of radians for equi-angular fan beam data. */

#ifdef RECON_ANGLE_FAN
PUBLIC double AngleBackprojectPoint
( 
 double x,          /* X position in 2D image space */
 double y,          /* Y position in 2D image space */
 double *qtau,      /* spacing of projection elements */
 double *ftau,      /* spacing of reconstructed image elements */
 double sin_t,      /* sin of angle made by projection with +X axis */
 double cos_t,      /* cos of angle made by projection with +X axis */
 double D           /* distance between origin and focal point */
)
{ /* AngleBackprojectPoint() */
/*  double l2;      square distance A+B declared static at top of file
                    since it is re-used in L2() */
    double cos_g;                        /* cos(gamma) */

  { /* calculate L2 */
    double X=x-D*cos_t; /* dummy variables used to calculate distance L2 */
    double Y=y-D*sin_t;

    l2=X*X+Y*Y;
  } /* calculate L2 */

  cos_g = (l2+D*D-(x*x+y*y))/(2.0*SQRT(l2)*D);

  if((cos_g>1.0)||(cos_g< -1.0)) /* needed for roundoff error in acos(cos_g) */
    cos_g = SIGN(cos_g);

  return(SIGN(x*sin_t-y*cos_t)*acos(cos_g));
} /* Anglebackprojectpoint() */
#endif RECON_ANGLE_FAN

/*****************************************************************************/

/* this function is used by BackprojectProjection() for equiangular fan beam
   data for the normalization factor needed.  It returns the squared distance
   between the focal point in equiangular fan bean data and a point (x,y).

   The return value is precalculated in AngleBackprojectPoint() where the
   focal point is converted to cartesian coordinates and the distance is
   computed directly. */

#ifdef RECON_ANGLE_FAN
PUBLIC double L2
(
 double D,      /* distance between center of rotation and focal point */
 double x,      /* X coordinate of point */
 double y,      /* Y coordinate of point */
 double sin_t,  /* sin(angle of center ray of projection) */
 double cos_t   /* cos(angle of center ray of projection) */
)
{ /* L2() */
  return(l2);  /* precalculated in AngleBackprojectPoint() */
} /* L2() */
#endif RECON_ANGLE_FAN
